#include <fstream>
#include <iostream>
#include "Trie.hpp"

bool is_word(const std::string& word) {
   for (char ch: word) {
      if (!((ch >= 'a' && ch <= 'z') || (ch >= 'A' && ch <= 'Z'))) {
	 return false;
      }
   }
   return true;
}

void visit(const std::string& word) {
   std::cout << word << std::endl;
}

void longest_path(Trie<std::string>::Pointer p, unsigned int count,
      unsigned int& maxcount, Trie<std::string>::Pointer& maxseen) {
   if (!p.defined()) return;
   if (p.exists()) {
      ++count;
      if (count > maxcount) {
	 maxcount = count; maxseen = p;
      }
   }
   for (char ch: p) {
      auto ptr = p;
      ptr.descend(ch);
      longest_path(ptr, count, maxcount, maxseen);
   }
}

int main() {
   Trie<std::string> trie;
   std::ifstream words("/usr/dict/words");
   if (!words) {
      std::cout << "unable to open /usr/dict/words. Bye" << std::endl; return 1;
   }
   std::string line;
   while (std::getline(words, line)) {
      if (!is_word(line)) continue;
      trie.insert(line, line);
   }
   std::string key;
   while (std::cin >> key) {
      if (is_word(key)) {
	 auto ptr = trie.descend();
	 if (ptr.defined()) {
	    for (char ch: key) {
	       if (!ptr.descend(ch)) break;
	       if (ptr.exists()) {
		  std::cout << *ptr << std::endl;
	       }
	    }
	    if (ptr.defined()) {
	       std::cout << "continuation possible with: ";
	       for (char ch: ptr) {
		  std::cout << ch;
	       }
	       std::cout << std::endl;
	    }
	 }
      }
   }
   Trie<std::string>::Pointer maxpath;
   unsigned int maxcount = 0;
   longest_path(trie.descend(), 0, maxcount, maxpath);
   if (maxcount > 0) {
      std::cout << *maxpath << std::endl;
   }
}
